<?php

class PlateauSquadro {
    const BLANC_V_ALLER = [1, 3, 2, 3, 1];
    const BLANC_V_RETOUR = [3, 1, 2, 1, 3];
    const NOIR_V_ALLER = [3, 1, 2, 1, 3];
    const NOIR_V_RETOUR = [1, 3, 2, 3, 1];

    private $plateau = [];
    private $lignesJouables = [1, 2, 3, 4, 5];
    private $colonnesJouables = [1, 2, 3, 4, 5];

    public function __construct() {
        $this->initCasesVides();
        $this->initCasesNeutres();
        $this->initCasesBlanches();
        $this->initCasesNoires();
    }

    private function initCasesVides() {
        for ($i = 0; $i < 7; $i++) {
            $this->plateau[$i] = new ArrayPieceSquadro();
            for ($j = 0; $j < 7; $j++) {
                $this->plateau[$i]->add(PieceSquadro::initVide());
            }
        }
    }

    private function initCasesNeutres() {
        $this->plateau[0][0] = PieceSquadro::initVide();
        $this->plateau[0][6] = PieceSquadro::initVide();
        $this->plateau[6][0] = PieceSquadro::initVide();
        $this->plateau[6][6] = PieceSquadro::initVide();
    }

    private function initCasesBlanches() {
        for ($i = 1; $i <= 5; $i++) {
            $this->plateau[$i][0] = PieceSquadro::initBlancOuest();
        }
    }

    private function initCasesNoires() {
        for ($j = 1; $j <= 5; $j++) {
            $this->plateau[6][$j] = PieceSquadro::initNoirSud();
        }
    }

    public function getPlateau() {
        return $this->plateau;
    }

    public function getPiece($x, $y) {
        return $this->plateau[$x][$y];
    }

    public function setPiece($piece, $x, $y) {
        $this->plateau[$x][$y] = $piece;
    }

    public function getLignesJouables() {
        return $this->lignesJouables;
    }

    public function getColonnesJouables() {
        return $this->colonnesJouables;
    }

    // Méthode retireLigneJouable et retireColonneJouable : Ces méthodes ne vérifient pas si l'index existe avant de tenter de le supprimer. Cela pourrait causer des erreurs si l'index est invalide.
    // Solution : Ajouter une vérification pour s'assurer que l'index existe avant de le supprimer.
    public function retireLigneJouable($index) {
        if (($key = array_search($index, $this->lignesJouables)) !== false) {
            unset($this->lignesJouables[$key]);
        } else {
            throw new OutOfBoundsException("Index de ligne invalide.");
        }
    }

    public function retireColonneJouable($index) {
        if (($key = array_search($index, $this->colonnesJouables)) !== false) {
            unset($this->colonnesJouables[$key]);
        } else {
            throw new OutOfBoundsException("Index de colonne invalide.");
        }
    }

    //Méthode getCoordDestination : La méthode ne vérifie pas si les coordonnées (x, y) sont valides avant de tenter de récupérer la pièce. Cela pourrait causer des erreurs si les coordonnées sont hors limites.
    //Solution : Ajouter une vérification pour s'assurer que les coordonnées sont valides.
    public function getCoordDestination($x, $y) {
        $piece = $this->getPiece($x, $y);
        if ($piece->getCouleur() === PieceSquadro::BLANC) {
            $direction = $piece->getDirection();
            $speed = ($direction === PieceSquadro::OUEST) ? self::BLANC_V_ALLER[$y - 1] : self::BLANC_V_RETOUR[$y - 1];
            return [$x, $y + $speed];
        } else {
            $direction = $piece->getDirection();
            $speed = ($direction === PieceSquadro::SUD) ? self::NOIR_V_ALLER[$x - 1] : self::NOIR_V_RETOUR[$x - 1];
            return [$x + $speed, $y];
        }
    }

    // Méthode getDestination : La méthode ne vérifie pas si les coordonnées de destination sont valides avant de tenter de récupérer la pièce. Cela pourrait causer des erreurs si les coordonnées sont hors limites.
    //Solution : Ajouter une vérification pour s'assurer que les coordonnées de destination sont valides.
    public function getDestination($x, $y) {
        list($newX, $newY) = $this->getCoordDestination($x, $y);
        if ($newX < 0 || $newX >= 7 || $newY < 0 || $newY >= 7) {
            throw new OutOfBoundsException("Coordonnées de destination invalides.");
        }
        return $this->getPiece($newX, $newY);
    }

    // Méthode toJson : La méthode ne vérifie pas si le plateau est valide avant de tenter de le sérialiser en JSON. Cela pourrait causer des erreurs si le plateau est malformé.
    // Solution : Ajouter une vérification pour s'assurer que le plateau est valide avant de le sérialiser.
    public function toJson() {
        return json_encode(array_map(function($row) {
            return $row->toJson();
        }, $this->plateau));
    }

    public static function fromJson($json) {
        $data = json_decode($json, true);
        $plateau = new self();
        foreach ($data as $i => $rowJson) {
            $plateau->plateau[$i] = ArrayPieceSquadro::fromJson($rowJson);
        }
        return $plateau;
    }
}
?>
